var teamspeak = require('node-teamspeak');
var events = require('events');
var util = require('util');

function Alfred() {
    var self = this;
    var eventMap = {'message': 'textmessage', 'join': 'cliententerview', 'leave': 'clientleftview'};
    var eventMapInv = {};
    
    self.config = {};
    self.stack = [];
    self.globals = {};
    self.connected = false;
    
    Alfred.prototype.login = function(options, callbackFunction) {
        var self = this;
        self.config = {
            'name': 'Alfred',
            'host': '127.0.0.1',
            'port': 10011,
            'loginName': 'serveradmin',
            'loginPass': 'login-pass',
            'sid': 1,
            'timeout': 999999999999
        };
        
        for(var prop in options) {
            self.config[prop] = options[prop];
        }
        
        self.teamspeak = new teamspeak(self.config.host, self.config.port);
        self.teamspeak.setTimeout(self.config.timeout);
        
        self.teamspeak.on('timeout', function() {
            console.log(require('moment')().add(1, 'hours').format('HH:mm:ss'), 'timeout');
        });
        
        self.teamspeak.send('login', {'client_login_name': self.config.loginName, 'client_login_password': self.config.loginPass}, function(err, res, raw) {
            if(err) throw new Error(util.inspect(err));
            
            self.teamspeak.send('use', {'sid': self.config.sid}, function(err, res, raw) {
                if(err) throw new Error(util.inspect(err));
                
                self.sendCommand = self.teamspeak.send;
                
                for(var i=0; i<self.stack.length; i++) if(typeof self.stack[i].callback.login === 'function') self.stack[i].callback.login.call(self);
                
                self.teamspeak.send('whoami', null, function(err, res, raw) {
                    if(err) throw new Error(util.inspect(err));
                    self.teamspeak.send('clientupdate', {'client_nickname': self.config.name});
                    self.clid = res['client_id'];
                });
                
                for(var eventAlias in eventMap) {
                    var curr = eventAlias;
                    
                    eventMapInv[eventMap[curr]] = curr;
                    
                    self.teamspeak.on(eventMap[eventAlias], function(res) {
                        var event = eventMapInv[Object.keys(res)[0]];
                        
                        self.callStack(event, res);
                    });
                }
                
                self.callStack('login', this);
                
                self.connected = true;
                if(typeof callbackFunction == 'function') callbackFunction.call(this);
            });
        });
    }
    
    Alfred.prototype.use = function() {
        var self = this;
        var args = Array.prototype.slice.call(arguments);
        var event = typeof args[0] === 'string' ? args.shift() : '<all>';
        
        for(var i=0; i<args.length; i++) {
            var extension = args[i];

            if(extension.init) extension.init.call(self);
        
            self.stack.push({
                'event': event,
                'callback': extension
            });
        }
    }
    
    Alfred.prototype.set = function(key, value) {
        return this.globals[key] = value;
    }
    
    Alfred.prototype.get = function(key) {
        return this.globals[key];
    }
    
    Alfred.prototype.callStack = function(event, data, callbackFunction) {
        var self = this;
        var stackController = new events.EventEmitter();
        var globalData = data;
        
        stackController.on('next', function(i) {
            if(i > self.stack.length - 1) {
                if(typeof callbackFunction == 'function') return callbackFunction();
                return;
            }

            if((self.stack[i].event != event && self.stack[i].event != '<all>') || typeof self.stack[i].callback !== 'function') return stackController.emit('next', i + 1);;
            
            self.stack[i].callback.call(self, event, globalData, function() {
                stackController.emit('next', i + 1);
            });
        });
        
        stackController.emit('next', 0); 
    }
}

module.exports = function() {
    var retAlfred = new Alfred();

    retAlfred.use(module.exports.query);
    retAlfred.use(module.exports.User);
    retAlfred.use('message', module.exports.cmd);
    
    return retAlfred;
};

module.exports.Base = function() {
    return new Alfred();    
};

module.exports.query = require('./lib/query.js');
module.exports.User = require('./lib/user.js');
module.exports.cmd = require('./lib/cmd.js');