var EventEmitter = require('events').EventEmitter;

var privateCommands = {};
var globalCommands = {};
var stack= {};

module.exports = function(event, data, next) {
    var bot = this;
    
    if(event != 'message' || bot.clid == data.user.info.clid) return next();
    
    var params = bot.parseCmd(data.msg);
	var paramCommand = params.shift();
    
    switch(parseInt(data.targetmode)) {
        case 1:
            for(var command in privateCommands) {
                if(privateCommands[command].regexp) {
                    if(privateCommands[command].regexp.test(paramCommand)) return callStack(privateCommands[command], data.user, params);
                } else if(paramCommand === bot.get('command identifier') + command) return callStack(privateCommands[command], data.user, params);
            }
            break;
        
        case 3:
            for(var command in globalCommands) {
                if(globalCommands[command].regexp) {
                    if(globalCommands[command].regexp.test(paramCommand)) return callStack(globalCommands[command], data.user, params);
                } else if(paramCommand === bot.get('command identifier') + command) return callStack(globalCommands[command], data.user, params);
            }
            break;
    }
    
    next();
}

module.exports.init = function() {
    this.set('command identifier', '!');
    
    this.privateCmd = function(cmd) {
        var args = Array.prototype.slice.call(arguments);
        args.splice(0, 1);
        
        privateCommands[cmd] = {};
        privateCommands[cmd].stack = args;
        privateCommands[cmd].regexp = cmd instanceof RegExp ? cmd : false;
        return this;
    }
    
    this.globalCmd = function(cmd) {
        var args = Array.prototype.slice.call(arguments);
        args.splice(0, 1);
        
        globalCommands[cmd] = {};
        globalCommands[cmd].stack = args;
        globalCommands[cmd].regexp = cmd instanceof RegExp ? cmd : false;
        return this;
    }
    
    this.parseCmd = function(cmd) {
    	var args = cmd.split(' ');
    	var indexStart = null, indexEnd = null;
    
    	for(var i=0; i<args.length; i++) {
    		if(args[i][0] == '"' && indexStart == null) indexStart = i;
    		if(args[i][args[i].length - 1] == '"' && indexStart != null) indexEnd = i;
    
    		if(indexStart != null && indexEnd != null) {
    			var argSplit = [];
    			for(var x=indexStart; x<=indexEnd; x++) {
    				if(x == indexStart) args[x] = args[x].substring(1, args[x].length);
    				if(x == indexEnd) args[x] = args[x].substring(0, args[x].length - 1);
    				argSplit.push(args[x]);
    			}
    			for(var x=indexStart + 1; x<indexEnd + 1; x++) {
    				args.splice(indexStart + 1, 1);
    			}
    			args[indexStart] = argSplit.join(' ');
    			i = i - indexEnd + 1;
    			indexStart = null;
    			indexEnd = null;
    		}
    	}
    
    	return args;
    }
}

module.exports.privateCommands = privateCommands;
module.exports.globalCommands = globalCommands;
module.exports.stack = stack;

function callStack(cmd, User, params, callbackFunction) {
    var stackController = new EventEmitter();
    
    stackController.on('next', function(i) {
        if(i > cmd.stack.length - 1) return callbackFunction();

        cmd.stack[i](User, params, function() {
            stackController.emit('next', i + 1);
        });
    });
    
    stackController.emit('next', 0);
}